@@ -228,11 +228,6 @@ int atag_arm_load(struct kexec_info *info, unsigned long base,
 	struct tag *params;
 	
 	buf = xmalloc(getpagesize());
-	if (!buf) {
-		fprintf(stderr, "Compiling ATAGs: out of memory\n");
-		return -1;
-	}
-
 	memset(buf, 0xff, getpagesize());
 	params = (struct tag *)buf;
 
@@ -315,8 +310,6 @@ static int setup_dtb_prop(char **bufp, off_t *sizep, int parentoffset,
 		dtb_size += fdt_node_len(node_name);
 		fdt_set_totalsize(dtb_buf, dtb_size);
 		dtb_buf = xrealloc(dtb_buf, dtb_size);
-		if (dtb_buf == NULL)
-			die("xrealloc failed\n");
 		off = fdt_add_subnode(dtb_buf, parentoffset, node_name);
 	}
 
@@ -340,8 +333,6 @@ static int setup_dtb_prop(char **bufp, off_t *sizep, int parentoffset,
 	if (fdt_totalsize(dtb_buf) < dtb_size) {
 		fdt_set_totalsize(dtb_buf, dtb_size);
 		dtb_buf = xrealloc(dtb_buf, dtb_size);
-		if (dtb_buf == NULL)
-			die("xrealloc failed\n");
 	}
 
 	if (fdt_setprop(dtb_buf, off, prop_name,
@@ -418,7 +409,7 @@ int zImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 		{ "page-offset",	1, 0, OPT_PAGE_OFFSET },
 		{ 0, 			0, 0, 0 },
 	};
-	static const char short_options[] = KEXEC_ARCH_OPT_STR "a:r:";
+	static const char short_options[] = KEXEC_ARCH_OPT_STR "";
 
 	/*
 	 * Parse the command line arguments
@@ -478,7 +469,7 @@ int zImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 			command_line_len = COMMAND_LINE_SIZE;
 	}
 	if (ramdisk)
-		ramdisk_buf = slurp_file(ramdisk, &initrd_size);
+		ramdisk_buf = slurp_file_mmap(ramdisk, &initrd_size);
 
 	if (dtb_file)
 		dtb_buf = slurp_file(dtb_file, &dtb_length);
@@ -552,6 +543,14 @@ int zImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 	 */
 	kernel_mem_size = len + 4;
 
+	/*
+	 * Check for a kernel size extension, and set or validate the
+	 * image size.  This is the total space needed to avoid the
+	 * boot kernel BSS, so other data (such as initrd) does not get
+	 * overwritten.
+	 */
+	tag = find_extension_tag(buf, len, ZIMAGE_TAG_KRNL_SIZE);
+
 	/*
 	 * The zImage length does not include its stack (4k) or its
 	 * malloc space (64k).  Include this.
@@ -560,13 +559,6 @@ int zImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 
 	dbgprintf("zImage requires 0x%08llx bytes\n", (unsigned long long)len);
 
-	/*
-	 * Check for a kernel size extension, and set or validate the
-	 * image size.  This is the total space needed to avoid the
-	 * boot kernel BSS, so other data (such as initrd) does not get
-	 * overwritten.
-	 */
-	tag = find_extension_tag(buf, len, ZIMAGE_TAG_KRNL_SIZE);
 	if (tag) {
 		uint32_t *p = (void *)buf + le32_to_cpu(tag->u.krnl_size.size_ptr);
 		uint32_t edata_size = le32_to_cpu(get_unaligned(p));
@@ -616,9 +608,6 @@ int zImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 		uint64_t start, end;
 
 		modified_cmdline = xmalloc(COMMAND_LINE_SIZE);
-		if (!modified_cmdline)
-			return -1;
-
 		memset(modified_cmdline, '\0', COMMAND_LINE_SIZE);
 
 		if (command_line) {
